/* $Id$ */

import java.io.*;
import java.util.*;
import java.awt.image.*;

import org.apache.pdfbox.exceptions.*;
import org.apache.pdfbox.pdmodel.*;
import org.apache.pdfbox.util.*;

import javax.imageio.*;
import javax.imageio.stream.*;
import javax.imageio.metadata.*;

/**
 * Test scan from Xerox WorkCentre 5030. sakelley@ucsd.edu
 */
public class TestRender {
    File infile;
    File outfile;
    private PDDocument document;
    private int imageType = BufferedImage.TYPE_INT_RGB;
    private int resolutionDPI = 96;
    private float imageQuality = 0.75f;
    
    public TestRender(String filename,String outdir) {
        infile = new File(filename);
        outfile = new File(outdir,infile.getName().replaceAll("\\.pdf","")+".jpeg");
    }
    
    public void renderInternal() throws IOException {
        System.out.println();
        System.out.println();
        System.out.println("Rendering "+infile+" to "+outfile);
        document = PDDocument.load(infile);
        List pages = document.getDocumentCatalog().getAllPages();
        PDPage page = (PDPage)(pages.get(0));
        BufferedImage image = page.convertToImage(imageType,resolutionDPI);

        BufferedOutputStream outStream = new BufferedOutputStream(
            new FileOutputStream(outfile));
        ImageOutputStream imageOutStream 
            = ImageIO.createImageOutputStream(outStream);
        ImageWriter iWriter = null;
        try {
            iWriter = ImageIO.getImageWritersByFormatName("JPEG").next();
            ImageWriteParam params = iWriter.getDefaultWriteParam();
            
            params.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
            params.setCompressionType("JPEG");
            params.setCompressionQuality(imageQuality);

            iWriter.setOutput(imageOutStream);
            
            IIOMetadata meta = null;
            
            // object to aggregate image, thumbnails, and metadata
            IIOImage iioImage = new IIOImage(image,null,meta);
            
            iWriter.write(
                null,           // optional stream metadata. null=use default
                iioImage,       // object w/image, thumbnails, and metadata
                params);        // params for writing process
        } finally {
            if (iWriter!=null) iWriter.dispose();
            document.close();
            outStream.close();
        }

    }

    public void render() {
        try {
            renderInternal();
        } catch(Throwable t) {
            t.printStackTrace();
        }
    }
    public static void main(String[] args) {
        String headless = System.getProperty("java.awt.headless");
        System.out.println("In TestRender.java, java.awt.headless = "+headless);
        
        try {
            String outdir = args[0];
            
            // File exported as PDF from OmniGraffle 5.2.3 (OS X 10.6)
            new TestRender("pdf/ItDoesntWorkExport.pdf",outdir).render();
            
            // File printed as PDF (OS X 10.6)
            new TestRender("pdf/ItDoesntWorkPrinted.pdf",outdir).render();
            
            // File printed on paper, then scanned w/Xerox WorkCentre 5030
            new TestRender("pdf/ItDoesntWorkScan.pdf",outdir).render();
            
            // This code, printed as PDF from BBEdit 9.6.3 (OS X 10.6)
            new TestRender("pdf/TestRender.pdf",outdir).render();

            // Converted to PDF/X3 w/ColorSync utility
            new TestRender("pdf/ItDoesntWorkScanX3.pdf",outdir).render();

            // Converted to PDF/A-1 w/Acrobat 9 Pro
            new TestRender("pdf/ItDoesntWorkScan_AdobeA-1B.pdf",outdir).render();

            // Converted to PDF/X3 w/Acrobat 9 Pro
            // Oddly, this causes intermittent failures, either on this
            // render, or a future one.
            // TIFFFaxDecoder5 at
            // org.apache.pdfbox.filter.TIFFFaxDecoder.decodeT6(TIFFFaxDecoder.java:1071)
            // When this one renders, it comes out more or less correct (I think)
            new TestRender("pdf/ItDoesntWorkScan_AdobeX3.pdf",outdir).render();

            // Copy of above file...
            new TestRender("pdf/ItDoesntWorkScan_AdobeA-1Bcopy2.pdf",outdir).render();

        } catch(Throwable t) {
            t.printStackTrace();
        }
    }
    
}
